#
# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# SPDX-License-Identifier: Apache-2.0
from unittest import mock

import testtools

import bandit
from bandit.core import constants
from bandit.core import issue


class IssueTests(testtools.TestCase):
    def test_issue_create(self):
        new_issue = _get_issue_instance()
        self.assertIsInstance(new_issue, issue.Issue)

    def test_issue_str(self):
        test_issue = _get_issue_instance()
        expect = (
            "Issue: 'Test issue' from B999:bandit_plugin:"
            " CWE: %s,"
            " Severity: MEDIUM "
            "Confidence: MEDIUM at code.py:1:8"
        )

        self.assertEqual(
            expect % str(issue.Cwe(issue.Cwe.MULTIPLE_BINDS)), str(test_issue)
        )

    def test_issue_as_dict(self):
        test_issue = _get_issue_instance()
        test_issue_dict = test_issue.as_dict(with_code=False)
        self.assertIsInstance(test_issue_dict, dict)
        self.assertEqual("code.py", test_issue_dict["filename"])
        self.assertEqual("bandit_plugin", test_issue_dict["test_name"])
        self.assertEqual("B999", test_issue_dict["test_id"])
        self.assertEqual("MEDIUM", test_issue_dict["issue_severity"])
        self.assertEqual(
            {
                "id": 605,
                "link": "https://cwe.mitre.org/data/definitions/605.html",
            },
            test_issue_dict["issue_cwe"],
        )
        self.assertEqual("MEDIUM", test_issue_dict["issue_confidence"])
        self.assertEqual("Test issue", test_issue_dict["issue_text"])
        self.assertEqual(1, test_issue_dict["line_number"])
        self.assertEqual([], test_issue_dict["line_range"])
        self.assertEqual(8, test_issue_dict["col_offset"])
        self.assertEqual(16, test_issue_dict["end_col_offset"])

    def test_issue_filter_severity(self):
        levels = [bandit.LOW, bandit.MEDIUM, bandit.HIGH]
        issues = [_get_issue_instance(level, bandit.HIGH) for level in levels]

        for level in levels:
            rank = constants.RANKING.index(level)
            for i in issues:
                test = constants.RANKING.index(i.severity)
                result = i.filter(level, bandit.UNDEFINED)
                self.assertTrue((test >= rank) == result)

    def test_issue_filter_confidence(self):
        levels = [bandit.LOW, bandit.MEDIUM, bandit.HIGH]
        issues = [_get_issue_instance(bandit.HIGH, level) for level in levels]

        for level in levels:
            rank = constants.RANKING.index(level)
            for i in issues:
                test = constants.RANKING.index(i.confidence)
                result = i.filter(bandit.UNDEFINED, level)
                self.assertTrue((test >= rank) == result)

    def test_matches_issue(self):
        issue_a = _get_issue_instance()

        issue_b = _get_issue_instance(severity=bandit.HIGH)

        issue_c = _get_issue_instance(confidence=bandit.LOW)

        issue_d = _get_issue_instance()
        issue_d.text = "ABCD"

        issue_e = _get_issue_instance()
        issue_e.fname = "file1.py"

        issue_f = issue_a

        issue_g = _get_issue_instance()
        issue_g.test = "ZZZZ"

        issue_h = issue_a
        issue_h.lineno = 12345

        # positive tests
        self.assertEqual(issue_a, issue_a)
        self.assertEqual(issue_a, issue_f)
        self.assertEqual(issue_f, issue_a)

        # severity doesn't match
        self.assertNotEqual(issue_a, issue_b)

        # confidence doesn't match
        self.assertNotEqual(issue_a, issue_c)

        # text doesn't match
        self.assertNotEqual(issue_a, issue_d)

        # filename doesn't match
        self.assertNotEqual(issue_a, issue_e)

        # plugin name doesn't match
        self.assertNotEqual(issue_a, issue_g)

        # line number doesn't match but should pass because we don't test that
        self.assertEqual(issue_a, issue_h)

    @mock.patch("linecache.getline")
    def test_get_code(self, getline):
        getline.return_value = b"\x08\x30"
        new_issue = issue.Issue(
            bandit.MEDIUM, cwe=issue.Cwe.MULTIPLE_BINDS, lineno=1
        )

        try:
            new_issue.get_code()
        except UnicodeDecodeError:
            self.fail("Bytes not properly decoded in issue.get_code()")


def _get_issue_instance(
    severity=bandit.MEDIUM,
    cwe=issue.Cwe.MULTIPLE_BINDS,
    confidence=bandit.MEDIUM,
):
    new_issue = issue.Issue(severity, cwe, confidence, "Test issue")
    new_issue.fname = "code.py"
    new_issue.test = "bandit_plugin"
    new_issue.test_id = "B999"
    new_issue.lineno = 1
    new_issue.col_offset = 8
    new_issue.end_col_offset = 16

    return new_issue



#
# Copyright 2015 Hewlett-Packard Enterprise
#
# SPDX-License-Identifier: Apache-2.0
import os
import subprocess
from unittest import mock

import fixtures
import git
import testtools

from bandit.cli import baseline

config = """
include:
    - '*.py'
    - '*.pyw'

profiles:
    test:
        include:
            - start_process_with_a_shell

shell_injection:
    subprocess: []
    no_shell: []
    shell:
        - os.system
"""


class BanditBaselineToolTests(testtools.TestCase):
    @classmethod
    def setUpClass(cls):
        # Set up prior to running test class
        # read in content used for temporary file contents
        with open("examples/mktemp.py") as fd:
            cls.temp_file_contents = fd.read()

    def setUp(self):
        # Set up prior to run each test case
        super().setUp()
        self.current_directory = os.getcwd()

    def tearDown(self):
        # Tear down after running each test case
        super().tearDown()
        os.chdir(self.current_directory)

    def test_bandit_baseline(self):
        # Tests running bandit via the CLI (baseline) with benign and malicious
        # content
        repo_directory = self.useFixture(fixtures.TempDir()).path

        # get benign and findings examples
        with open("examples/okay.py") as fd:
            benign_contents = fd.read()

        with open("examples/os_system.py") as fd:
            malicious_contents = fd.read()

        contents = {
            "benign_one.py": benign_contents,
            "benign_two.py": benign_contents,
            "malicious.py": malicious_contents,
        }

        # init git repo, change directory to it
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial commit")
        os.chdir(repo_directory)

        with open("bandit.yaml", "w") as fd:
            fd.write(config)

        # create three branches, first has only benign, second adds malicious,
        # third adds benign

        branches = [
            {
                "name": "benign1",
                "files": ["benign_one.py"],
                "expected_return": 0,
            },
            {
                "name": "malicious",
                "files": ["benign_one.py", "malicious.py"],
                "expected_return": 1,
            },
            {
                "name": "benign2",
                "files": ["benign_one.py", "malicious.py", "benign_two.py"],
                "expected_return": 0,
            },
        ]

        baseline_command = [
            "bandit-baseline",
            "-c",
            "bandit.yaml",
            "-r",
            ".",
            "-p",
            "test",
        ]

        for branch in branches:
            branch["branch"] = git_repo.create_head(branch["name"])
            git_repo.head.reference = branch["branch"]
            git_repo.head.reset(working_tree=True)

            for f in branch["files"]:
                with open(f, "w") as fd:
                    fd.write(contents[f])

            git_repo.index.add(branch["files"])
            git_repo.index.commit(branch["name"])

            self.assertEqual(
                branch["expected_return"], subprocess.call(baseline_command)
            )

    def test_main_non_repo(self):
        # Test that bandit gracefully exits when there is no git repository
        # when calling main
        repo_dir = self.useFixture(fixtures.TempDir()).path
        os.chdir(repo_dir)

        # assert the system exits with code 2
        self.assertRaisesRegex(SystemExit, "2", baseline.main)

    def test_main_git_command_failure(self):
        # Test that bandit does not run when the Git command fails
        repo_directory = self.useFixture(fixtures.TempDir()).path
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        additional_content = "additional_file.py"
        with open(additional_content, "w") as fd:
            fd.write(self.temp_file_contents)
        git_repo.index.add([additional_content])
        git_repo.index.commit("Additional Content")

        with mock.patch("git.Repo.commit") as mock_git_repo_commit:
            mock_git_repo_commit.side_effect = git.exc.GitCommandError(
                "commit", ""
            )

            # assert the system exits with code 2
            self.assertRaisesRegex(SystemExit, "2", baseline.main)

    def test_main_no_parent_commit(self):
        # Test that bandit exits when there is no parent commit detected when
        # calling main
        repo_directory = self.useFixture(fixtures.TempDir()).path

        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        # assert the system exits with code 2
        self.assertRaisesRegex(SystemExit, "2", baseline.main)

    def test_main_subprocess_error(self):
        # Test that bandit handles a CalledProcessError when attempting to run
        # bandit baseline via a subprocess
        repo_directory = self.useFixture(fixtures.TempDir()).path

        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        additional_content = "additional_file.py"
        with open(additional_content, "w") as fd:
            fd.write(self.temp_file_contents)
        git_repo.index.add([additional_content])
        git_repo.index.commit("Additional Content")

        with mock.patch("subprocess.check_output") as mock_check_output:
            mock_bandit_cmd = "bandit_mock -b temp_file.txt"
            mock_check_output.side_effect = subprocess.CalledProcessError(
                "3", mock_bandit_cmd
            )

            # assert the system exits with code 3 (returned from
            # CalledProcessError)
            self.assertRaisesRegex(SystemExit, "3", baseline.main)

    def test_init_logger(self):
        # Test whether the logger was initialized when calling init_logger
        baseline.init_logger()
        logger = baseline.LOG

        # verify that logger was initialized
        self.assertIsNotNone(logger)

    def test_initialize_no_repo(self):
        # Test that bandit does not run when there is no current git
        # repository when calling initialize
        repo_directory = self.useFixture(fixtures.TempDir()).path
        os.chdir(repo_directory)

        return_value = baseline.initialize()

        # assert bandit did not run due to no git repo
        self.assertEqual((None, None, None), return_value)

    def test_initialize_git_command_failure(self):
        # Test that bandit does not run when the Git command fails
        repo_directory = self.useFixture(fixtures.TempDir()).path
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        additional_content = "additional_file.py"
        with open(additional_content, "w") as fd:
            fd.write(self.temp_file_contents)
        git_repo.index.add([additional_content])
        git_repo.index.commit("Additional Content")

        with mock.patch("git.Repo") as mock_git_repo:
            mock_git_repo.side_effect = git.exc.GitCommandNotFound("clone", "")

            return_value = baseline.initialize()

            # assert bandit did not run due to git command failure
            self.assertEqual((None, None, None), return_value)

    def test_initialize_dirty_repo(self):
        # Test that bandit does not run when the current git repository is
        # 'dirty' when calling the initialize method
        repo_directory = self.useFixture(fixtures.TempDir()).path
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        # make the git repo 'dirty'
        with open("dirty_file.py", "w") as fd:
            fd.write(self.temp_file_contents)
        git_repo.index.add(["dirty_file.py"])

        return_value = baseline.initialize()

        # assert bandit did not run due to dirty repo
        self.assertEqual((None, None, None), return_value)

    @mock.patch("sys.argv", ["bandit", "-f", "txt", "test"])
    def test_initialize_existing_report_file(self):
        # Test that bandit does not run when the output file exists (and the
        # provided output format does not match the default format) when
        # calling the initialize method
        repo_directory = self.useFixture(fixtures.TempDir()).path
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        # create an existing version of output report file
        existing_report = f"{baseline.report_basename}.txt"
        with open(existing_report, "w") as fd:
            fd.write(self.temp_file_contents)

        return_value = baseline.initialize()

        # assert bandit did not run due to existing report file
        self.assertEqual((None, None, None), return_value)

    @mock.patch(
        "bandit.cli.baseline.bandit_args", ["-o", "bandit_baseline_result"]
    )
    def test_initialize_with_output_argument(self):
        # Test that bandit does not run when the '-o' (output) argument is
        # specified
        repo_directory = self.useFixture(fixtures.TempDir()).path
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        return_value = baseline.initialize()

        # assert bandit did not run due to provided -o (--ouput) argument
        self.assertEqual((None, None, None), return_value)

    def test_initialize_existing_temp_file(self):
        # Test that bandit does not run when the temporary output file exists
        # when calling the initialize method
        repo_directory = self.useFixture(fixtures.TempDir()).path
        git_repo = git.Repo.init(repo_directory)
        git_repo.index.commit("Initial Commit")
        os.chdir(repo_directory)

        # create an existing version of temporary output file
        existing_temp_file = baseline.baseline_tmp_file
        with open(existing_temp_file, "w") as fd:
            fd.write(self.temp_file_contents)

        return_value = baseline.initialize()

        # assert bandit did not run due to existing temporary report file
        self.assertEqual((None, None, None), return_value)



from cryptography.hazmat import backends
from cryptography.hazmat.primitives.asymmetric import dsa
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric import rsa
from Crypto.PublicKey import DSA as pycrypto_dsa
from Crypto.PublicKey import RSA as pycrypto_rsa
from Cryptodome.PublicKey import DSA as pycryptodomex_dsa
from Cryptodome.PublicKey import RSA as pycryptodomex_rsa


# Correct
dsa.generate_private_key(key_size=2048,
                         backend=backends.default_backend())
ec.generate_private_key(curve=ec.SECP384R1,
                        backend=backends.default_backend())
rsa.generate_private_key(public_exponent=65537,
                         key_size=2048,
                         backend=backends.default_backend())
pycrypto_dsa.generate(bits=2048)
pycrypto_rsa.generate(bits=2048)
pycryptodomex_dsa.generate(bits=2048)
pycryptodomex_rsa.generate(bits=2048)

# Also correct: without keyword args
dsa.generate_private_key(4096,
                         backends.default_backend())
ec.generate_private_key(ec.SECP256K1,
                        backends.default_backend())
rsa.generate_private_key(3,
                         4096,
                         backends.default_backend())
pycrypto_dsa.generate(4096)
pycrypto_rsa.generate(4096)
pycryptodomex_dsa.generate(4096)
pycryptodomex_rsa.generate(4096)

# Incorrect: weak key sizes
dsa.generate_private_key(key_size=1024,
                         backend=backends.default_backend())
ec.generate_private_key(curve=ec.SECT163R2,
                        backend=backends.default_backend())
rsa.generate_private_key(public_exponent=65537,
                         key_size=1024,
                         backend=backends.default_backend())
pycrypto_dsa.generate(bits=1024)
pycrypto_rsa.generate(bits=1024)
pycryptodomex_dsa.generate(bits=1024)
pycryptodomex_rsa.generate(bits=1024)

# Also incorrect: without keyword args
dsa.generate_private_key(512,
                         backends.default_backend())
ec.generate_private_key(ec.SECT163R2,
                        backends.default_backend())
rsa.generate_private_key(3,
                         512,
                         backends.default_backend())
pycrypto_dsa.generate(512)
pycrypto_rsa.generate(512)
pycryptodomex_dsa.generate(512)
pycryptodomex_rsa.generate(512)

# Don't crash when the size is variable
rsa.generate_private_key(public_exponent=65537,
                         key_size=some_key_size,
                         backend=backends.default_backend())

# Can't reliably know which curve was passed, in some cases like below
ec.generate_private_key(
    curve=curves[self.curve]['create'](self.size),
    backend=backends.default_backend()
)



def subprocess_shell_cmd():
    # sample function with known subprocess shell cmd candidates

def yaml_load():
    # sample function with known yaml.load candidates

def xml_sax_make_parser():
    # sample function with known xml.sax.make_parser candidates



import os
from django.utils import safestring


def insecure_function(text, cls=''):
    return '<h1 class="{cls}">{text}</h1>'.format(text=text, cls=cls)


my_insecure_str = insecure_function('insecure', cls='" onload="alert(\'xss\')')
safestring.mark_safe(my_insecure_str)
safestring.SafeText(my_insecure_str)
safestring.SafeUnicode(my_insecure_str)
safestring.SafeString(my_insecure_str)
safestring.SafeBytes(my_insecure_str)


def try_insecure(cls='" onload="alert(\'xss\')'):
    try:
        my_insecure_str = insecure_function('insecure', cls=cls)
    except Exception:
        my_insecure_str = 'Secure'
    safestring.mark_safe(my_insecure_str)


def except_insecure(cls='" onload="alert(\'xss\')'):
    try:
        my_insecure_str = 'Secure'
    except Exception:
        my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe(my_insecure_str)


def try_else_insecure(cls='" onload="alert(\'xss\')'):
    try:
        if 1 == random.randint(0, 1):  # nosec
            raise Exception
    except Exception:
        my_insecure_str = 'Secure'
    else:
        my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe(my_insecure_str)


def finally_insecure(cls='" onload="alert(\'xss\')'):
    try:
        if 1 == random.randint(0, 1):  # nosec
            raise Exception
    except Exception:
        print("Exception")
    else:
        print("No Exception")
    finally:
        my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe(my_insecure_str)


def format_arg_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>{} {}</b>'.format(my_insecure_str, 'STR'))


def format_startarg_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>{}</b>'.format(*[my_insecure_str]))


def format_keywords_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>{b}</b>'.format(b=my_insecure_str))


def format_kwargs_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>{b}</b>'.format(**{'b': my_insecure_str}))


def percent_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>%s</b>' % my_insecure_str)


def percent_list_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>%s %s</b>' % (my_insecure_str, 'b'))


def percent_dict_insecure(cls='" onload="alert(\'xss\')'):
    my_insecure_str = insecure_function('insecure', cls=cls)
    safestring.mark_safe('<b>%(b)s</b>' % {'b': my_insecure_str})


def import_insecure():
    import sre_constants
    safestring.mark_safe(sre_constants.ANY)


def import_as_insecure():
    import sre_constants.ANY as any_str
    safestring.mark_safe(any_str)


def from_import_insecure():
    from sre_constants import ANY
    safestring.mark_safe(ANY)


def from_import_as_insecure():
    from sre_constants import ANY as any_str
    safestring.mark_safe(any_str)


def with_insecure(path):
    with open(path) as f:
        safestring.mark_safe(f.read())


def also_with_insecure(path):
    with open(path) as f:
        safestring.mark_safe(f)


def for_insecure():
    my_secure_str = ''
    for i in range(random.randint(0, 1)):  # nosec
        my_secure_str += insecure_function('insecure', cls='" onload="alert(\'xss\')')
    safestring.mark_safe(my_secure_str)


def while_insecure():
    my_secure_str = ''
    while ord(os.urandom(1)) % 2 == 0:
        my_secure_str += insecure_function('insecure', cls='" onload="alert(\'xss\')')
    safestring.mark_safe(my_secure_str)


def some_insecure_case():
    if ord(os.urandom(1)) % 2 == 0:
        my_secure_str = insecure_function('insecure', cls='" onload="alert(\'xss\')')
    elif ord(os.urandom(1)) % 2 == 0:
        my_secure_str = 'Secure'
    else:
        my_secure_str = 'Secure'
    safestring.mark_safe(my_secure_str)

mystr = 'insecure'


def test_insecure_shadow():  # var assigned out of scope
    safestring.mark_safe(mystr)


def test_insecure(str_arg):
    safestring.mark_safe(str_arg)


def test_insecure_with_assign(str_arg=None):
    if not str_arg:
        str_arg = 'could be insecure'
    safestring.mark_safe(str_arg)

def test_insecure_tuple_assign():
    HTML_CHOICES = (
        (_('Donate'), 'https://example.org/donate/'),
        (_('More info'), 'https://example.org/'),
    )
    text, url = choice(HTML_CHOICES)
    safestring.mark_safe('<a href="{0}">{1}</a>'.format(url, text))



import requests
import wsgiref.handlers

def application(environ, start_response):
    r = requests.get('https://192.168.0.42/private/api/foobar', timeout=30)
    start_response('200 OK', [('Content-Type', 'text/plain')])
    return [r.content]

if __name__ == '__main__':
    wsgiref.handlers.CGIHandler().run(application)



import flask
from markupsafe import Markup, escape

content = "<script>alert('Hello, world!')</script>"
Markup(f"unsafe {content}")  # B704
flask.Markup("unsafe {}".format(content))  # B704
Markup("safe {}").format(content)
flask.Markup(b"safe {}", encoding='utf-8').format(content)
escape(content)
Markup(content)  # B704
flask.Markup("unsafe %s" % content)  # B704
Markup(object="safe")
Markup(object="unsafe {}".format(content))  # Not currently detected



from twisted.internet import reactor
from twisted.web import static, server, twcgi

root = static.File("/root")
root.putChild("cgi-bin", twcgi.CGIDirectory("/var/www/cgi-bin"))
reactor.listenTCP(80, server.Site(root))
reactor.run()



#!/usr/bin/env python3
# cf. https://trojansource.codes/ & https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2021-42574
access_level = "user"
if access_level != 'none‮⁦': # Check if admin ⁩⁦' and access_level != 'user
    print("You are an admin.\n")



from bleach import clean
from markupsafe import Markup

content = "<script>alert('Hello, world!')</script>"
Markup(clean(content))

# indirect assignments are currently not supported
cleaned = clean(content)
Markup(cleaned)



import torch
import torchvision.models as models

# Example of saving a model
model = models.resnet18(pretrained=True)
torch.save(model.state_dict(), 'model_weights.pth')

# Example of loading the model weights in an insecure way (should trigger B614)
loaded_model = models.resnet18()
loaded_model.load_state_dict(torch.load('model_weights.pth'))

# Example of loading with weights_only=True (should NOT trigger B614)
safe_model = models.resnet18()
safe_model.load_state_dict(torch.load('model_weights.pth', weights_only=True))

# Example of loading with weights_only=False (should trigger B614)
unsafe_model = models.resnet18()
unsafe_model.load_state_dict(torch.load('model_weights.pth', weights_only=False))

# Example of loading with map_location but no weights_only (should trigger B614)
cpu_model = models.resnet18()
cpu_model.load_state_dict(torch.load('model_weights.pth', map_location='cpu'))

# Example of loading with both map_location and weights_only=True (should NOT trigger B614)
safe_cpu_model = models.resnet18()
safe_cpu_model.load_state_dict(torch.load('model_weights.pth', map_location='cpu', weights_only=True))



# Copyright (c) 2015 VMware, Inc.
# Copyright (c) 2015 Hewlett Packard Enterprise
#
# SPDX-License-Identifier: Apache-2.0
import collections
import tempfile
from unittest import mock

import testtools

import bandit
from bandit.core import config
from bandit.core import docs_utils
from bandit.core import issue
from bandit.core import manager
from bandit.formatters import screen


class ScreenFormatterTests(testtools.TestCase):
    def setUp(self):
        super().setUp()

    @mock.patch("bandit.core.issue.Issue.get_code")
    def test_output_issue(self, get_code):
        issue = _get_issue_instance()
        get_code.return_value = "DDDDDDD"
        indent_val = "CCCCCCC"

        def _template(_issue, _indent_val, _code, _color):
            return_val = [
                "{}{}>> Issue: [{}:{}] {}".format(
                    _indent_val,
                    _color,
                    _issue.test_id,
                    _issue.test,
                    _issue.text,
                ),
                "{}   Severity: {}   Confidence: {}".format(
                    _indent_val,
                    _issue.severity.capitalize(),
                    _issue.confidence.capitalize(),
                ),
                f"{_indent_val}   CWE: {_issue.cwe}",
                f"{_indent_val}   More Info: "
                f"{docs_utils.get_url(_issue.test_id)}",
                "{}   Location: {}:{}:{}{}".format(
                    _indent_val,
                    _issue.fname,
                    _issue.lineno,
                    _issue.col_offset,
                    screen.COLOR["DEFAULT"],
                ),
            ]
            if _code:
                return_val.append(f"{_indent_val}{_code}")
            return "\n".join(return_val)

        issue_text = screen._output_issue_str(issue, indent_val)
        expected_return = _template(
            issue, indent_val, "DDDDDDD", screen.COLOR["MEDIUM"]
        )
        self.assertEqual(expected_return, issue_text)

        issue_text = screen._output_issue_str(
            issue, indent_val, show_code=False
        )
        expected_return = _template(
            issue, indent_val, "", screen.COLOR["MEDIUM"]
        )
        self.assertEqual(expected_return, issue_text)

        issue.lineno = ""
        issue.col_offset = ""
        issue_text = screen._output_issue_str(
            issue, indent_val, show_lineno=False
        )
        expected_return = _template(
            issue, indent_val, "DDDDDDD", screen.COLOR["MEDIUM"]
        )
        self.assertEqual(expected_return, issue_text)

    @mock.patch("bandit.core.manager.BanditManager.get_issue_list")
    def test_no_issues(self, get_issue_list):
        conf = config.BanditConfig()
        self.manager = manager.BanditManager(conf, "file")

        (tmp_fd, self.tmp_fname) = tempfile.mkstemp()
        self.manager.out_file = self.tmp_fname

        get_issue_list.return_value = collections.OrderedDict()
        with mock.patch("bandit.formatters.screen.do_print") as m:
            with open(self.tmp_fname, "w") as tmp_file:
                screen.report(
                    self.manager, tmp_file, bandit.LOW, bandit.LOW, lines=5
                )
            self.assertIn(
                "No issues identified.",
                "\n".join([str(a) for a in m.call_args]),
            )

    @mock.patch("bandit.core.manager.BanditManager.get_issue_list")
    def test_report_nobaseline(self, get_issue_list):
        conf = config.BanditConfig()
        self.manager = manager.BanditManager(conf, "file")

        (tmp_fd, self.tmp_fname) = tempfile.mkstemp()
        self.manager.out_file = self.tmp_fname

        self.manager.verbose = True
        self.manager.files_list = ["binding.py"]

        self.manager.scores = [
            {"SEVERITY": [0, 0, 0, 1], "CONFIDENCE": [0, 0, 0, 1]}
        ]

        self.manager.skipped = [("abc.py", "File is bad")]
        self.manager.excluded_files = ["def.py"]

        issue_a = _get_issue_instance()
        issue_b = _get_issue_instance()

        get_issue_list.return_value = [issue_a, issue_b]

        self.manager.metrics.data["_totals"] = {"loc": 1000, "nosec": 50}
        for category in ["SEVERITY", "CONFIDENCE"]:
            for level in ["UNDEFINED", "LOW", "MEDIUM", "HIGH"]:
                self.manager.metrics.data["_totals"][f"{category}.{level}"] = 1

        # Validate that we're outputting the correct issues
        output_str_fn = "bandit.formatters.screen._output_issue_str"
        with mock.patch(output_str_fn) as output_str:
            output_str.return_value = "ISSUE_OUTPUT_TEXT"

            with open(self.tmp_fname, "w") as tmp_file:
                screen.report(
                    self.manager, tmp_file, bandit.LOW, bandit.LOW, lines=5
                )

            calls = [
                mock.call(issue_a, "", lines=5),
                mock.call(issue_b, "", lines=5),
            ]

            output_str.assert_has_calls(calls, any_order=True)

        # Validate that we're outputting all of the expected fields and the
        # correct values
        with mock.patch("bandit.formatters.screen.do_print") as m:
            with open(self.tmp_fname, "w") as tmp_file:
                screen.report(
                    self.manager, tmp_file, bandit.LOW, bandit.LOW, lines=5
                )

            data = "\n".join([str(a) for a in m.call_args[0][0]])

            expected = "Run started"
            self.assertIn(expected, data)

            expected_items = [
                screen.header("Files in scope (1):"),
                "\n\tbinding.py (score: {SEVERITY: 1, CONFIDENCE: 1})",
            ]

            for item in expected_items:
                self.assertIn(item, data)

            expected = screen.header("Files excluded (1):") + "\n\tdef.py"
            self.assertIn(expected, data)

            expected = (
                "Total lines of code: 1000\n\tTotal lines skipped "
                "(#nosec): 50"
            )
            self.assertIn(expected, data)

            expected = (
                "Total issues (by severity):\n\t\tUndefined: 1\n\t\t"
                "Low: 1\n\t\tMedium: 1\n\t\tHigh: 1"
            )
            self.assertIn(expected, data)

            expected = (
                "Total issues (by confidence):\n\t\tUndefined: 1\n\t\t"
                "Low: 1\n\t\tMedium: 1\n\t\tHigh: 1"
            )
            self.assertIn(expected, data)

            expected = (
                screen.header("Files skipped (1):")
                + "\n\tabc.py (File is bad)"
            )
            self.assertIn(expected, data)

    @mock.patch("bandit.core.manager.BanditManager.get_issue_list")
    def test_report_baseline(self, get_issue_list):
        conf = config.BanditConfig()
        self.manager = manager.BanditManager(conf, "file")

        (tmp_fd, self.tmp_fname) = tempfile.mkstemp()
        self.manager.out_file = self.tmp_fname

        issue_a = _get_issue_instance()
        issue_b = _get_issue_instance()

        issue_x = _get_issue_instance()
        issue_x.fname = "x"
        issue_y = _get_issue_instance()
        issue_y.fname = "y"
        issue_z = _get_issue_instance()
        issue_z.fname = "z"

        get_issue_list.return_value = collections.OrderedDict(
            [(issue_a, [issue_x]), (issue_b, [issue_y, issue_z])]
        )

        # Validate that we're outputting the correct issues
        indent_val = " " * 10
        output_str_fn = "bandit.formatters.screen._output_issue_str"
        with mock.patch(output_str_fn) as output_str:
            output_str.return_value = "ISSUE_OUTPUT_TEXT"

            with open(self.tmp_fname, "w") as tmp_file:
                screen.report(
                    self.manager, tmp_file, bandit.LOW, bandit.LOW, lines=5
                )

            calls = [
                mock.call(issue_a, "", lines=5),
                mock.call(issue_b, "", show_code=False, show_lineno=False),
                mock.call(issue_y, indent_val, lines=5),
                mock.call(issue_z, indent_val, lines=5),
            ]

            output_str.assert_has_calls(calls, any_order=True)


def _get_issue_instance(
    severity=bandit.MEDIUM, cwe=123, confidence=bandit.MEDIUM
):
    new_issue = issue.Issue(severity, cwe, confidence, "Test issue")
    new_issue.fname = "code.py"
    new_issue.test = "bandit_plugin"
    new_issue.lineno = 1
    return new_issue



#
# Copyright 2014 Hewlett-Packard Development Company, L.P.
# Copyright 2015 Nebula, Inc.
#
# SPDX-License-Identifier: Apache-2.0
import ast
import os
import shutil
import sys
import tempfile

import testtools

from bandit.core import utils as b_utils


def _touch(path):
    """Create an empty file at ``path``."""
    open(path, "w").close()


class UtilTests(testtools.TestCase):
    """This set of tests exercises bandit.core.util functions."""

    def setUp(self):
        super().setUp()
        self._setup_get_module_qualname_from_path()

    def _setup_get_module_qualname_from_path(self):
        """Setup a fake directory for testing get_module_qualname_from_path().

        Create temporary directory and then create fake .py files
        within directory structure.  We setup test cases for
        a typical module, a path misssing a middle __init__.py,
        no __init__.py anywhere in path, symlinking .py files.
        """

        self.tempdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tempdir)
        self.reltempdir = os.path.relpath(self.tempdir)

        # good/a/b/c/test_typical.py
        os.makedirs(os.path.join(self.tempdir, "good", "a", "b", "c"), 0o755)
        _touch(os.path.join(self.tempdir, "good", "__init__.py"))
        _touch(os.path.join(self.tempdir, "good", "a", "__init__.py"))
        _touch(os.path.join(self.tempdir, "good", "a", "b", "__init__.py"))
        _touch(
            os.path.join(self.tempdir, "good", "a", "b", "c", "__init__.py")
        )
        _touch(
            os.path.join(
                self.tempdir, "good", "a", "b", "c", "test_typical.py"
            )
        )

        # missingmid/a/b/c/test_missingmid.py
        os.makedirs(
            os.path.join(self.tempdir, "missingmid", "a", "b", "c"), 0o755
        )
        _touch(os.path.join(self.tempdir, "missingmid", "__init__.py"))
        # no missingmid/a/__init__.py
        _touch(
            os.path.join(self.tempdir, "missingmid", "a", "b", "__init__.py")
        )
        _touch(
            os.path.join(
                self.tempdir, "missingmid", "a", "b", "c", "__init__.py"
            )
        )
        _touch(
            os.path.join(
                self.tempdir, "missingmid", "a", "b", "c", "test_missingmid.py"
            )
        )

        # missingend/a/b/c/test_missingend.py
        os.makedirs(
            os.path.join(self.tempdir, "missingend", "a", "b", "c"), 0o755
        )
        _touch(os.path.join(self.tempdir, "missingend", "__init__.py"))
        _touch(
            os.path.join(self.tempdir, "missingend", "a", "b", "__init__.py")
        )
        # no missingend/a/b/c/__init__.py
        _touch(
            os.path.join(
                self.tempdir, "missingend", "a", "b", "c", "test_missingend.py"
            )
        )

        # syms/a/bsym/c/test_typical.py
        os.makedirs(os.path.join(self.tempdir, "syms", "a"), 0o755)
        _touch(os.path.join(self.tempdir, "syms", "__init__.py"))
        _touch(os.path.join(self.tempdir, "syms", "a", "__init__.py"))
        os.symlink(
            os.path.join(self.tempdir, "good", "a", "b"),
            os.path.join(self.tempdir, "syms", "a", "bsym"),
        )

    def test_get_module_qualname_from_path_abs_typical(self):
        """Test get_module_qualname_from_path with typical absolute paths."""

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.tempdir, "good", "a", "b", "c", "test_typical.py"
            )
        )
        self.assertEqual("good.a.b.c.test_typical", name)

    def test_get_module_qualname_from_path_with_dot(self):
        """Test get_module_qualname_from_path with a "." ."""

        name = b_utils.get_module_qualname_from_path(
            os.path.join(".", "__init__.py")
        )

        self.assertEqual("__init__", name)

    def test_get_module_qualname_from_path_abs_missingmid(self):
        # Test get_module_qualname_from_path with missing module
        # __init__.py

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.tempdir, "missingmid", "a", "b", "c", "test_missingmid.py"
            )
        )
        self.assertEqual("b.c.test_missingmid", name)

    def test_get_module_qualname_from_path_abs_missingend(self):
        # Test get_module_qualname_from_path with no __init__.py
        # last dir'''

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.tempdir, "missingend", "a", "b", "c", "test_missingend.py"
            )
        )
        self.assertEqual("test_missingend", name)

    def test_get_module_qualname_from_path_abs_syms(self):
        """Test get_module_qualname_from_path with symlink in path."""

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.tempdir, "syms", "a", "bsym", "c", "test_typical.py"
            )
        )
        self.assertEqual("syms.a.bsym.c.test_typical", name)

    def test_get_module_qualname_from_path_rel_typical(self):
        """Test get_module_qualname_from_path with typical relative paths."""

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.reltempdir, "good", "a", "b", "c", "test_typical.py"
            )
        )
        self.assertEqual("good.a.b.c.test_typical", name)

    def test_get_module_qualname_from_path_rel_missingmid(self):
        # Test get_module_qualname_from_path with module __init__.py
        # missing and relative paths

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.reltempdir,
                "missingmid",
                "a",
                "b",
                "c",
                "test_missingmid.py",
            )
        )
        self.assertEqual("b.c.test_missingmid", name)

    def test_get_module_qualname_from_path_rel_missingend(self):
        # Test get_module_qualname_from_path with __init__.py missing from
        # last dir and using relative paths

        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.reltempdir,
                "missingend",
                "a",
                "b",
                "c",
                "test_missingend.py",
            )
        )
        self.assertEqual("test_missingend", name)

    def test_get_module_qualname_from_path_rel_syms(self):
        """Test get_module_qualname_from_path with symbolic relative paths."""
        name = b_utils.get_module_qualname_from_path(
            os.path.join(
                self.reltempdir, "syms", "a", "bsym", "c", "test_typical.py"
            )
        )
        self.assertEqual("syms.a.bsym.c.test_typical", name)

    def test_get_module_qualname_from_path_sys(self):
        """Test get_module_qualname_from_path with system module paths."""

        name = b_utils.get_module_qualname_from_path(os.__file__)
        self.assertEqual("os", name)

        # This will fail because of magic for os.path. Not sure how to fix.
        # name = b_utils.get_module_qualname_from_path(os.path.__file__)
        # self.assertEqual(name, 'os.path')

    def test_get_module_qualname_from_path_invalid_path(self):
        """Test get_module_qualname_from_path with invalid path."""

        name = b_utils.get_module_qualname_from_path("/a/b/c/d/e.py")
        self.assertEqual("e", name)

    def test_get_module_qualname_from_path_dir(self):
        """Test get_module_qualname_from_path with dir path."""

        self.assertRaises(
            b_utils.InvalidModulePath,
            b_utils.get_module_qualname_from_path,
            "/tmp/",
        )

    def test_namespace_path_join(self):
        p = b_utils.namespace_path_join("base1.base2", "name")
        self.assertEqual("base1.base2.name", p)

    def test_namespace_path_split(self):
        (head, tail) = b_utils.namespace_path_split("base1.base2.name")
        self.assertEqual("base1.base2", head)
        self.assertEqual("name", tail)

    def test_get_call_name1(self):
        """Gets a qualified call name."""
        tree = ast.parse("a.b.c.d(x,y)").body[0].value
        name = b_utils.get_call_name(tree, {})
        self.assertEqual("a.b.c.d", name)

    def test_get_call_name2(self):
        """Gets qualified call name and resolves aliases."""
        tree = ast.parse("a.b.c.d(x,y)").body[0].value

        name = b_utils.get_call_name(tree, {"a": "alias.x.y"})
        self.assertEqual("alias.x.y.b.c.d", name)

        name = b_utils.get_call_name(tree, {"a.b": "alias.x.y"})
        self.assertEqual("alias.x.y.c.d", name)

        name = b_utils.get_call_name(tree, {"a.b.c.d": "alias.x.y"})
        self.assertEqual("alias.x.y", name)

    def test_get_call_name3(self):
        """Getting name for a complex call."""
        tree = ast.parse("a.list[0](x,y)").body[0].value
        name = b_utils._get_attr_qual_name(tree, {})
        self.assertEqual("", name)
        # TODO(ljfisher) At best we might be able to get:
        # self.assertEqual(name, 'a.list[0]')

    def test_linerange(self):
        with open("./examples/jinja2_templating.py") as test_file:
            tree = ast.parse(test_file.read())
        # Check linerange returns corrent number of lines
        line = tree.body[8]
        lrange = b_utils.linerange(line)

        # line 9 should be three lines long
        self.assertEqual(3, len(lrange))

        # the range should be the correct line numbers
        self.assertEqual([11, 12, 13], list(lrange))

    def test_path_for_function(self):
        path = b_utils.get_path_for_function(b_utils.get_path_for_function)
        self.assertEqual(path, b_utils.__file__)

    def test_path_for_function_no_file(self):
        self.assertIsNone(b_utils.get_path_for_function(sys.settrace))

    def test_path_for_function_no_module(self):
        self.assertIsNone(b_utils.get_path_for_function(1))

    def test_escaped_representation_simple(self):
        res = b_utils.escaped_bytes_representation(b"ascii")
        self.assertEqual(res, b"ascii")

    def test_escaped_representation_valid_not_printable(self):
        res = b_utils.escaped_bytes_representation(b"\\u0000")
        self.assertEqual(res, b"\\x00")

    def test_escaped_representation_invalid(self):
        res = b_utils.escaped_bytes_representation(b"\\uffff")
        self.assertEqual(res, b"\\uffff")

    def test_escaped_representation_mixed(self):
        res = b_utils.escaped_bytes_representation(b"ascii\\u0000\\uffff")
        self.assertEqual(res, b"ascii\\x00\\uffff")

    def test_deepgetattr(self):
        a = type("", (), {})
        a.b = type("", (), {})
        a.b.c = type("", (), {})
        a.b.c.d = "deep value"
        a.b.c.d2 = "deep value 2"
        a.b.c.e = "a.b.c"
        self.assertEqual("deep value", b_utils.deepgetattr(a.b.c, "d"))
        self.assertEqual("deep value 2", b_utils.deepgetattr(a.b.c, "d2"))
        self.assertEqual("a.b.c", b_utils.deepgetattr(a.b.c, "e"))
        self.assertEqual("deep value", b_utils.deepgetattr(a, "b.c.d"))
        self.assertEqual("deep value 2", b_utils.deepgetattr(a, "b.c.d2"))
        self.assertRaises(AttributeError, b_utils.deepgetattr, a.b, "z")

    def test_parse_ini_file(self):
        tests = [
            {
                "content": "[bandit]\nexclude=/abc,/def",
                "expected": {"exclude": "/abc,/def"},
            },
            {"content": "[Blabla]\nsomething=something", "expected": None},
        ]

        with tempfile.NamedTemporaryFile("r+") as t:
            for test in tests:
                with open(t.name, "w") as f:
                    f.write(test["content"])

                self.assertEqual(
                    b_utils.parse_ini_file(t.name), test["expected"]
                )

    def test_check_ast_node_good(self):
        node = b_utils.check_ast_node("Call")
        self.assertEqual("Call", node)

    def test_check_ast_node_bad_node(self):
        self.assertRaises(TypeError, b_utils.check_ast_node, "Derp")

    def test_check_ast_node_bad_type(self):
        self.assertRaises(TypeError, b_utils.check_ast_node, "walk")



from Crypto.Cipher import AES
from Crypto import Random

from . import CryptoMaterialsCacheEntry


def test_pycrypto():
    key = b'Sixteen byte key'
    iv = Random.new().read(AES.block_size)
    cipher = pycrypto_arc2.new(key, AES.MODE_CFB, iv)
    factory = CryptoMaterialsCacheEntry()



from paramiko import client
from paramiko import AutoAddPolicy
from paramiko import WarningPolicy

ssh_client = client.SSHClient()
ssh_client.set_missing_host_key_policy(client.AutoAddPolicy)
ssh_client.set_missing_host_key_policy(client.WarningPolicy)
ssh_client.set_missing_host_key_policy(client.AutoAddPolicy())
ssh_client.set_missing_host_key_policy(client.WarningPolicy())

ssh_client.set_missing_host_key_policy(AutoAddPolicy)
ssh_client.set_missing_host_key_policy(WarningPolicy)
ssh_client.set_missing_host_key_policy(AutoAddPolicy())
ssh_client.set_missing_host_key_policy(WarningPolicy())



from django.db.models.expressions import RawSQL
from django.contrib.auth.models import User

User.objects.annotate(val=RawSQL('secure', []))
User.objects.annotate(val=RawSQL('%secure' % 'nos', []))
User.objects.annotate(val=RawSQL('{}secure'.format('no'), []))
raw = '"username") AS "val" FROM "auth_user" WHERE "username"="admin" --'
User.objects.annotate(val=RawSQL(raw, []))
raw = '"username") AS "val" FROM "auth_user"' \
      ' WHERE "username"="admin" OR 1=%s --'
User.objects.annotate(val=RawSQL(raw, [0]))
User.objects.annotate(val=RawSQL(sql='{}secure'.format('no'), params=[]))
User.objects.annotate(val=RawSQL(params=[], sql='{}secure'.format('no')))



#
# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# SPDX-License-Identifier: Apache-2.0
import collections
import logging

LOG = logging.getLogger(__name__)


class BanditMetaAst:
    nodes = collections.OrderedDict()

    def __init__(self):
        pass

    def add_node(self, node, parent_id, depth):
        """Add a node to the AST node collection

        :param node: The AST node to add
        :param parent_id: The ID of the node's parent
        :param depth: The depth of the node
        :return: -
        """
        node_id = hex(id(node))
        LOG.debug("adding node : %s [%s]", node_id, depth)
        self.nodes[node_id] = {
            "raw": node,
            "parent_id": parent_id,
            "depth": depth,
        }

    def __str__(self):
        """Dumps a listing of all of the nodes

        Dumps a listing of all of the nodes for debugging purposes
        :return: -
        """
        tmpstr = ""
        for k, v in self.nodes.items():
            tmpstr += f"Node: {k}\n"
            tmpstr += f"\t{str(v)}\n"
        tmpstr += f"Length: {len(self.nodes)}\n"
        return tmpstr



import os
from django.utils import safestring

safestring.mark_safe('<b>secure</b>')
safestring.SafeText('<b>secure</b>')
safestring.SafeUnicode('<b>secure</b>')
safestring.SafeString('<b>secure</b>')
safestring.SafeBytes('<b>secure</b>')

my_secure_str = '<b>Hello World</b>'
safestring.mark_safe(my_secure_str)

my_secure_str, _ = ('<b>Hello World</b>', '')
safestring.mark_safe(my_secure_str)

also_secure_str = my_secure_str
safestring.mark_safe(also_secure_str)


def try_secure():
    try:
        my_secure_str = 'Secure'
    except Exception:
        my_secure_str = 'Secure'
    else:
        my_secure_str = 'Secure'
    finally:
        my_secure_str = 'Secure'
    safestring.mark_safe(my_secure_str)


def format_secure():
    safestring.mark_safe('<b>{}</b>'.format('secure'))
    my_secure_str = 'secure'
    safestring.mark_safe('<b>{}</b>'.format(my_secure_str))
    safestring.mark_safe('<b>{} {}</b>'.format(my_secure_str, 'a'))
    safestring.mark_safe('<b>{} {}</b>'.format(*[my_secure_str, 'a']))
    safestring.mark_safe('<b>{b}</b>'.format(b=my_secure_str))  # nosec TODO
    safestring.mark_safe('<b>{b}</b>'.format(**{'b': my_secure_str}))  # nosec TODO
    my_secure_str = '<b>{}</b>'.format(my_secure_str)
    safestring.mark_safe(my_secure_str)


def percent_secure():
    safestring.mark_safe('<b>%s</b>' % 'secure')
    my_secure_str = 'secure'
    safestring.mark_safe('<b>%s</b>' % my_secure_str)
    safestring.mark_safe('<b>%s %s</b>' % (my_secure_str, 'a'))
    safestring.mark_safe('<b>%(b)s</b>' % {'b': my_secure_str})  # nosec TODO


def with_secure(path):
    with open(path) as f:
        safestring.mark_safe('Secure')


def loop_secure():
    my_secure_str = ''

    for i in range(ord(os.urandom(1))):
        my_secure_str += ' Secure'
    safestring.mark_safe(my_secure_str)
    while ord(os.urandom(1)) % 2 == 0:
        my_secure_str += ' Secure'
    safestring.mark_safe(my_secure_str)


def all_secure_case():
    if ord(os.urandom(1)) % 2 == 0:
        my_secure_str = 'Secure'
    elif ord(os.urandom(1)) % 2 == 0:
        my_secure_str = 'Secure'
    else:
        my_secure_str = 'Secure'
    safestring.mark_safe(my_secure_str)



from cryptography.hazmat.primitives import hashes
from Crypto.Hash import MD2 as pycrypto_md2
from Crypto.Hash import MD4 as pycrypto_md4
from Crypto.Hash import MD5 as pycrypto_md5
from Crypto.Hash import SHA as pycrypto_sha
from Cryptodome.Hash import MD2 as pycryptodomex_md2
from Cryptodome.Hash import MD4 as pycryptodomex_md4
from Cryptodome.Hash import MD5 as pycryptodomex_md5
from Cryptodome.Hash import SHA as pycryptodomex_sha
import hashlib
import crypt

hashlib.md5(1)
hashlib.md5(1).hexdigest()

abc = str.replace(hashlib.md5("1"), "###")

print(hashlib.md5("1"))

hashlib.sha1(1)

hashlib.sha1(usedforsecurity=False)

pycrypto_md2.new()
pycrypto_md4.new()
pycrypto_md5.new()
pycrypto_sha.new()

pycryptodomex_md2.new()
pycryptodomex_md4.new()
pycryptodomex_md5.new()
pycryptodomex_sha.new()

hashes.MD5()
hashes.SHA1()

crypt.crypt("asdfasdfasdfasdf", salt=crypt.METHOD_CRYPT)
crypt.crypt("asdfasdfasdfasdf", salt=crypt.METHOD_MD5)
crypt.crypt("asdfasdfasdfasdf", salt=crypt.METHOD_BLOWFISH)
crypt.crypt("asdfasdfasdfasdf")
crypt.crypt("asdfasdfasdfasdf", salt=crypt.METHOD_SHA256)
crypt.crypt("asdfasdfasdfasdf", salt=crypt.METHOD_SHA512)

crypt.mksalt(crypt.METHOD_CRYPT)
crypt.mksalt(crypt.METHOD_MD5)
crypt.mksalt(crypt.METHOD_BLOWFISH)
crypt.mksalt()
crypt.mksalt(crypt.METHOD_SHA256)
crypt.mksalt(crypt.METHOD_SHA512)



import hashlib

hashlib.new('md5')

hashlib.new('md4', b'test')

hashlib.new(name='md5', data=b'test')

hashlib.new('MD4', data=b'test')

hashlib.new('sha1')

hashlib.new('sha1', data=b'test')

hashlib.new('sha', data=b'test')

hashlib.new(name='SHA', data=b'test')

# usedforsecurity arg only availabe in Python 3.9+
hashlib.new('sha1', usedforsecurity=True)

# Test that plugin does not flag valid hash functions.
hashlib.new('sha256')

hashlib.new('SHA512')

# usedforsecurity arg only availabe in Python 3.9+
hashlib.new(name='sha1', usedforsecurity=False)



#
# Copyright 2015 Red Hat, Inc.
#
# SPDX-License-Identifier: Apache-2.0
import ast
from unittest import mock

import testtools

from bandit.core import context


class ContextTests(testtools.TestCase):
    def test_context_create(self):
        ref_context = mock.Mock()
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(ref_context, new_context._context)

        new_context = context.Context()
        self.assertIsInstance(new_context._context, dict)

    def test_repr(self):
        ref_object = dict(spam="eggs")
        expected_repr = f"<Context {ref_object}>"
        new_context = context.Context(context_object=ref_object)
        self.assertEqual(expected_repr, repr(new_context))

    @mock.patch("bandit.core.context.Context._get_literal_value")
    def test_call_args(self, get_literal_value):
        get_literal_value.return_value = "eggs"
        ref_call = mock.Mock()
        ref_call.args = [mock.Mock(attr="spam"), "eggs"]
        ref_context = dict(call=ref_call)
        new_context = context.Context(context_object=ref_context)
        expected_args = ["spam", "eggs"]
        self.assertListEqual(expected_args, new_context.call_args)

    def test_call_args_count(self):
        ref_call = mock.Mock()
        ref_call.args = ["spam", "eggs"]
        ref_context = dict(call=ref_call)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(len(ref_call.args), new_context.call_args_count)

        ref_context = dict(call={})
        new_context = context.Context(context_object=ref_context)
        self.assertIsNone(new_context.call_args_count)

        new_context = context.Context()
        self.assertIsNone(new_context.call_args_count)

    def test_call_function_name(self):
        expected_string = "spam"
        ref_context = dict(name=expected_string)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(expected_string, new_context.call_function_name)

        new_context = context.Context()
        self.assertIsNone(new_context.call_function_name)

    def test_call_function_name_qual(self):
        expected_string = "spam"
        ref_context = dict(qualname=expected_string)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(expected_string, new_context.call_function_name_qual)

        new_context = context.Context()
        self.assertIsNone(new_context.call_function_name_qual)

    @mock.patch("bandit.core.context.Context._get_literal_value")
    def test_call_keywords(self, get_literal_value):
        get_literal_value.return_value = "eggs"
        ref_keyword1 = mock.Mock(arg="arg1", value=mock.Mock(attr="spam"))
        ref_keyword2 = mock.Mock(arg="arg2", value="eggs")
        ref_call = mock.Mock()
        ref_call.keywords = [ref_keyword1, ref_keyword2]
        ref_context = dict(call=ref_call)
        new_context = context.Context(context_object=ref_context)
        expected_dict = dict(arg1="spam", arg2="eggs")
        self.assertDictEqual(expected_dict, new_context.call_keywords)

        ref_context = dict(call=None)
        new_context = context.Context(context_object=ref_context)
        self.assertIsNone(new_context.call_keywords)

        new_context = context.Context()
        self.assertIsNone(new_context.call_keywords)

    def test_node(self):
        expected_node = "spam"
        ref_context = dict(node=expected_node)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(expected_node, new_context.node)

        new_context = context.Context()
        self.assertIsNone(new_context.node)

    def test_string_val(self):
        expected_string = "spam"
        ref_context = dict(str=expected_string)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(expected_string, new_context.string_val)

        new_context = context.Context()
        self.assertIsNone(new_context.string_val)

    def test_statement(self):
        expected_string = "spam"
        ref_context = dict(statement=expected_string)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(expected_string, new_context.statement)

        new_context = context.Context()
        self.assertIsNone(new_context.statement)

    @mock.patch("bandit.core.utils.get_qual_attr")
    def test_function_def_defaults_qual(self, get_qual_attr):
        get_qual_attr.return_value = "spam"
        ref_node = mock.Mock(args=mock.Mock(defaults=["spam"]))
        ref_context = dict(node=ref_node, import_aliases=None)
        new_context = context.Context(context_object=ref_context)
        self.assertListEqual(["spam"], new_context.function_def_defaults_qual)

        ref_node = mock.Mock(args=mock.Mock(defaults=[]))
        ref_context = dict(node=ref_node, import_aliases=None)
        new_context = context.Context(context_object=ref_context)
        self.assertListEqual([], new_context.function_def_defaults_qual)

        new_context = context.Context()
        self.assertListEqual([], new_context.function_def_defaults_qual)

    def test__get_literal_value(self):
        new_context = context.Context()

        value = ast.Num(42)
        expected = value.n
        self.assertEqual(expected, new_context._get_literal_value(value))

        value = ast.Str("spam")
        expected = value.s
        self.assertEqual(expected, new_context._get_literal_value(value))

        value = ast.List([ast.Str("spam"), ast.Num(42)], ast.Load())
        expected = [ast.Str("spam").s, ast.Num(42).n]
        self.assertListEqual(expected, new_context._get_literal_value(value))

        value = ast.Tuple([ast.Str("spam"), ast.Num(42)], ast.Load())
        expected = (ast.Str("spam").s, ast.Num(42).n)
        self.assertTupleEqual(expected, new_context._get_literal_value(value))

        value = ast.Set([ast.Str("spam"), ast.Num(42)])
        expected = {ast.Str("spam").s, ast.Num(42).n}
        self.assertSetEqual(expected, new_context._get_literal_value(value))

        value = ast.Dict(["spam", "eggs"], [42, "foo"])
        expected = dict(spam=42, eggs="foo")
        self.assertDictEqual(expected, new_context._get_literal_value(value))

        value = ast.Ellipsis()
        self.assertIsNone(new_context._get_literal_value(value))

        value = ast.Name("spam", ast.Load())
        expected = value.id
        self.assertEqual(expected, new_context._get_literal_value(value))

        value = ast.Bytes(b"spam")
        expected = value.s
        self.assertEqual(expected, new_context._get_literal_value(value))

        self.assertIsNone(new_context._get_literal_value(None))

    @mock.patch(
        "bandit.core.context.Context.call_keywords",
        new_callable=mock.PropertyMock,
    )
    def test_check_call_arg_value(self, call_keywords):
        new_context = context.Context()
        call_keywords.return_value = dict(spam="eggs")
        self.assertTrue(new_context.check_call_arg_value("spam", "eggs"))
        self.assertTrue(
            new_context.check_call_arg_value("spam", ["spam", "eggs"])
        )
        self.assertFalse(new_context.check_call_arg_value("spam", "spam"))
        self.assertFalse(new_context.check_call_arg_value("spam"))
        self.assertFalse(new_context.check_call_arg_value("eggs"))

        new_context = context.Context()
        self.assertIsNone(new_context.check_call_arg_value(None))

    @mock.patch(
        "bandit.core.context.Context.node", new_callable=mock.PropertyMock
    )
    def test_get_lineno_for_call_arg(self, node):
        expected_lineno = 42
        keyword1 = mock.Mock(
            arg="spam", value=mock.Mock(lineno=expected_lineno)
        )
        node.return_value = mock.Mock(keywords=[keyword1])
        new_context = context.Context()
        actual_lineno = new_context.get_lineno_for_call_arg("spam")
        self.assertEqual(expected_lineno, actual_lineno)

        new_context = context.Context()
        missing_lineno = new_context.get_lineno_for_call_arg("eggs")
        self.assertIsNone(missing_lineno)

    def test_get_call_arg_at_position(self):
        expected_arg = "spam"
        ref_call = mock.Mock()
        ref_call.args = [ast.Str(expected_arg)]
        ref_context = dict(call=ref_call)
        new_context = context.Context(context_object=ref_context)
        self.assertEqual(expected_arg, new_context.get_call_arg_at_position(0))
        self.assertIsNone(new_context.get_call_arg_at_position(1))

        ref_call = mock.Mock()
        ref_call.args = []
        ref_context = dict(call=ref_call)
        new_context = context.Context(context_object=ref_context)
        self.assertIsNone(new_context.get_call_arg_at_position(0))

        new_context = context.Context()
        self.assertIsNone(new_context.get_call_arg_at_position(0))

    def test_is_module_being_imported(self):
        ref_context = dict(module="spam")
        new_context = context.Context(context_object=ref_context)
        self.assertTrue(new_context.is_module_being_imported("spam"))
        self.assertFalse(new_context.is_module_being_imported("eggs"))

        new_context = context.Context()
        self.assertFalse(new_context.is_module_being_imported("spam"))

    def test_is_module_imported_exact(self):
        ref_context = dict(imports=["spam"])
        new_context = context.Context(context_object=ref_context)
        self.assertTrue(new_context.is_module_imported_exact("spam"))
        self.assertFalse(new_context.is_module_imported_exact("eggs"))

        new_context = context.Context()
        self.assertFalse(new_context.is_module_being_imported("spam"))

    def test_is_module_imported_like(self):
        ref_context = dict(imports=[["spam"], ["eggs"]])
        new_context = context.Context(context_object=ref_context)
        self.assertTrue(new_context.is_module_imported_like("spam"))
        self.assertFalse(new_context.is_module_imported_like("bacon"))

        new_context = context.Context()
        self.assertFalse(new_context.is_module_imported_like("spam"))

    def test_filename(self):
        ref_context = dict(filename="spam.py")
        new_context = context.Context(context_object=ref_context)

        self.assertEqual(new_context.filename, "spam.py")

        new_context = context.Context()
        self.assertIsNone(new_context.filename)



# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# SPDX-License-Identifier: Apache-2.0
import os

import setuptools


data_files = []
man_path = "doc/build/man/bandit.1"
if os.path.isfile(man_path):
    data_files.append(("share/man/man1", [man_path]))


setuptools.setup(
    python_requires=">=3.9",
    setup_requires=["pbr>=2.0.0"],
    pbr=True,
    data_files=data_files,
)



#
# Copyright 2016 Hewlett-Packard Development Company, L.P.
#
# SPDX-License-Identifier: Apache-2.0
import testtools

from bandit.core import blacklisting


class BlacklistingTests(testtools.TestCase):
    def test_report_issue(self):
        data = {"level": "HIGH", "message": "test {name}", "id": "B000"}

        issue = blacklisting.report_issue(data, "name")
        issue_dict = issue.as_dict(with_code=False)
        self.assertIsInstance(issue_dict, dict)
        self.assertEqual("B000", issue_dict["test_id"])
        self.assertEqual("HIGH", issue_dict["issue_severity"])
        self.assertEqual({}, issue_dict["issue_cwe"])
        self.assertEqual("HIGH", issue_dict["issue_confidence"])
        self.assertEqual("test name", issue_dict["issue_text"])

    def test_report_issue_defaults(self):
        data = {"message": "test {name}"}

        issue = blacklisting.report_issue(data, "name")
        issue_dict = issue.as_dict(with_code=False)
        self.assertIsInstance(issue_dict, dict)
        self.assertEqual("LEGACY", issue_dict["test_id"])
        self.assertEqual("MEDIUM", issue_dict["issue_severity"])
        self.assertEqual({}, issue_dict["issue_cwe"])
        self.assertEqual("HIGH", issue_dict["issue_confidence"])
        self.assertEqual("test name", issue_dict["issue_text"])



import os
import shelve
import tempfile

with tempfile.TemporaryDirectory() as d:
    filename = os.path.join(d, 'shelf')

    with shelve.open(filename) as db:
        db['spam'] = {'eggs': 'ham'}

    with shelve.open(filename) as db:
        print(db['spam'])



# SPDX-License-Identifier: Apache-2.0
from datetime import datetime
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join("..", "..")))
# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx_copybutton",
]

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# The suffix of source filenames.
source_suffix = ".rst"

# The root toctree document.
root_doc = "index"

# General information about the project.
project = "Bandit"
copyright = f"{datetime.now():%Y}, Bandit Developers"

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

modindex_common_prefix = ["bandit."]

# -- Options for man page output --------------------------------------------

# Grouping the document tree for man pages.
# List of tuples 'sourcefile', 'target', u'title', u'Authors name', 'manual'

man_pages = [
    (
        "man/bandit",
        "bandit",
        "Python source code security analyzer",
        ["PyCQA"],
        1,
    )
]

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
html_theme = "sphinx_rtd_theme"
# html_static_path = ['static']
html_theme_options = {}

# Output file base name for HTML help builder.
htmlhelp_basename = f"{project}doc"

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    (
        "index",
        f"{project}.tex",
        f"{project} Documentation",
        "PyCQA",
        "manual",
    ),
]

# Example configuration for intersphinx: refer to the Python standard library.
# intersphinx_mapping = {'http://docs.python.org/': None}



from datasets import load_dataset
from huggingface_hub import hf_hub_download, snapshot_download
from transformers import AutoModel, AutoTokenizer

# UNSAFE USAGE

# AutoModel (Model Loading)

# Example #1: No revision (defaults to floating 'main')
unsafe_model_no_revision = AutoModel.from_pretrained("org/model_name")

# Example #2: Floating revision: 'main'
unsafe_model_main = AutoModel.from_pretrained(
    "org/model_name",
    revision="main"
)

# Example #3: Floating tag revision: 'v1.0.0'
unsafe_model_tag = AutoModel.from_pretrained(
    "org/model_name",
    revision="v1.0.0"
)


# AutoTokenizer (Tokenizer Loading)

# Example #4: No revision
unsafe_tokenizer_no_revision = AutoTokenizer.from_pretrained("org/model_name")

# Example #5: Floating revision: 'main'
unsafe_tokenizer_main = AutoTokenizer.from_pretrained(
    "org/model_name",
    revision="main"
)

# Example #6: Floating tag revision: 'v1.0.0'
unsafe_tokenizer_tag = AutoTokenizer.from_pretrained(
    "org/model_name",
    revision="v1.0.0"
)


# Example #7: load_dataset (Dataset Loading)

# Example #8: No revision
unsafe_dataset_no_revision = load_dataset("org_dataset")

# Example #9: Floating revision: 'main'
unsafe_dataset_main = load_dataset("org_dataset", revision="main")

# Example #10: Floating tag revision: 'v1.0.0'
unsafe_dataset_tag = load_dataset("org_dataset", revision="v1.0.0")


# f_hub_download (File Download)

# Example #11: No revision
unsafe_file_no_revision = hf_hub_download(
    repo_id="org/model_name",
    filename="config.json"
)

# Example #12: Floating revision: 'main'
unsafe_file_main = hf_hub_download(
    repo_id="org/model_name",
    filename="config.json",
    revision="main"
)

# Example #13: Floating tag revision: 'v1.0.0'
unsafe_file_tag = hf_hub_download(
    repo_id="org/model_name",
    filename="config.json",
    revision="v1.0.0"
)


# snapshot_download (Repo Snapshot)

# Example #14: No revision
unsafe_snapshot_no_revision = snapshot_download(repo_id="org/model_name")

# Example #15: Floating revision: 'main'
unsafe_snapshot_main = snapshot_download(
    repo_id="org/model_name",
    revision="main"
)

# Example #16: Floating tag revision: 'v1.0.0'
unsafe_snapshot_tag = snapshot_download(
    repo_id="org/model_name",
    revision="v1.0.0"
)


# -------------------------------
# SAFE USAGE
# -------------------------------

# AutoModel

# Example #17: Pinned commit hash
safe_model_commit = AutoModel.from_pretrained(
    "org/model_name",
    revision="5d0f2e8a7f1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d"
)

# Example #18: Local path
safe_model_local = AutoModel.from_pretrained("./local_model")
safe_model_local_abs = AutoModel.from_pretrained("/path/to/model")

# AutoTokenizer

# Example #19: Pinned commit hash
safe_tokenizer_commit = AutoTokenizer.from_pretrained(
    "org/model_name",
    revision="5d0f2e8a7f1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d"
)

# Example #20: Local path
safe_tokenizer_local = AutoTokenizer.from_pretrained("./local_tokenizer")


# load_dataset

# Example #21: Pinned commit hash
safe_dataset_commit = load_dataset(
    "org_dataset",
    revision="5d0f2e8a7f1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d"
)


# hf_hub_download

# Example #22: Pinned commit hash
safe_file_commit = hf_hub_download(
    repo_id="org/model_name",
    filename="config.json",
    revision="5d0f2e8a7f1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d"
)


# snapshot_download

# Example #23: Pinned commit hash
safe_snapshot_commit = snapshot_download(
    repo_id="org/model_name",
    revision="5d0f2e8a7f1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d"
)
